@@ -872,9 +872,11 @@ function validate_password($password) {
     $result = array();
     $val_conf = Config::read_array('password_validation');
 
-    $minlen = (int) Config::read_string('min_password_length'); # used up to 2.3.x - check it for backward compatibility
-    if ($minlen > 0) {
-        $val_conf['/.{' . $minlen . '}/'] = "password_too_short $minlen";
+    if (Config::has('min_password_length')) {
+        $minlen = (int)Config::read_string('min_password_length'); # used up to 2.3.x - check it for backward compatibility
+        if ($minlen > 0) {
+            $val_conf['/.{' . $minlen . '}/'] = "password_too_short $minlen";
+        }
     }
 
     foreach ($val_conf as $regex => $message) {
@@ -1557,6 +1559,47 @@ $DEBUG_TEXT = <<<EOF
     </ul>
 EOF;
 
+
+/**
+ * @return string - PDO DSN for PHP.
+ * @throws Exception
+ */
+function db_connection_string() {
+    global $CONF;
+    $dsn = null;
+    if (db_mysql()) {
+        $socket = false;
+        if (Config::has('database_socket')) {
+            $socket = Config::read_string('database_socket');
+        }
+
+        $database_name = Config::read_string('database_name');
+
+        if ($socket) {
+            $dsn = "mysql:unix_socket={$socket};dbname={$database_name};charset=UTF8";
+        } else {
+            $dsn = "mysql:host={$CONF['database_host']};dbname={$database_name};charset=UTF8";
+        }
+    } elseif (db_sqlite()) {
+        $db = $CONF['database_name'];
+
+        $dsn = "sqlite:{$db}";
+    } elseif (db_pgsql()) {
+        $dsn = "pgsql:dbname={$CONF['database_name']}";
+        if (isset($CONF['database_host'])) {
+            $dsn .= ";host={$CONF['database_host']}";
+        }
+        if (isset($CONF['database_port'])) {
+            $dsn .= ";port={$CONF['database_port']}";
+        }
+        $dsn .= ";options='-c client_encoding=utf8'";
+    } else {
+        throw new Exception("<p style='color: red'>FATAL Error:<br />Invalid \$CONF['database_type'] <br/>'pgsql', 'mysql' or 'sqlite' supported. <br/> Please fix your config.inc.php!</p>");
+    }
+
+    return $dsn;
+}
+
 /**
  * db_connect
  * Action: Makes a connection to the database if it doesn't exist
@@ -1577,24 +1620,19 @@ function db_connect() {
 
     $link = false;
 
+    // throws.
+    $dsn = db_connection_string();
+
     $options = array(
         PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
+        PDO::ATTR_TIMEOUT => 5,
     );
     $username_password = true;
 
     $queries = array();
 
-    $dsn = null;
 
     if (db_mysql()) {
-        $socket = Config::read_string('database_socket');
-        $database_name = Config::read_string('database_name');
-
-        if ($socket) {
-            $dsn = "mysql:unix_socket={$socket};dbname={$database_name};charset=UTF8";
-        } else {
-            $dsn = "mysql:host={$CONF['database_host']};dbname={$database_name};charset=UTF8";
-        }
         if (Config::bool('database_use_ssl')) {
             $options[PDO::MYSQL_ATTR_SSL_KEY] = Config::read_string('database_ssl_key');
             $options[PDO::MYSQL_ATTR_SSL_CA] = Config::read_string('database_ssl_ca');
@@ -1616,31 +1654,23 @@ function db_connect() {
         $db = $CONF['database_name'];
 
         if (!file_exists($db)) {
-            $error_text = 'SQLite database missing: '. $db;
+            $error_text = 'SQLite database missing: ' . $db;
             throw new Exception($error_text);
         }
 
         if (!is_writeable($db)) {
-            $error_text = 'SQLite database not writeable: '. $db;
+            $error_text = 'SQLite database not writeable: ' . $db;
             throw new Exception($error_text);
         }
 
         if (!is_writeable(dirname($db))) {
-            $error_text = 'The directory the SQLite database is in is not writeable: '. dirname($db);
+            $error_text = 'The directory the SQLite database is in is not writeable: ' . dirname($db);
             throw new Exception($error_text);
         }
 
-        $dsn = "sqlite:{$db}";
         $username_password = false;
     } elseif (db_pgsql()) {
-        $dsn = "pgsql:dbname={$CONF['database_name']}";
-        if (isset($CONF['database_host'])) {
-            $dsn .= ";host={$CONF['database_host']}";
-        }
-        if (isset($CONF['database_port'])) {
-            $dsn .= ";port={$CONF['database_port']}";
-        }
-        $dsn .= ";options='-c client_encoding=utf8'";
+        // nothing to do.
     } else {
         throw new Exception("<p style='color: red'>FATAL Error:<br />Invalid \$CONF['database_type']! Please fix your config.inc.php!</p>");
     }
@@ -2085,8 +2115,6 @@ function table_by_key($table_key) {
     $table = $CONF['database_prefix'] . $table;
 
     if (db_mysql()) {
-        // try and ensure we don't get ``table`` ?
-        $table = preg_replace('/`/', '', $table);
         return "`" . $table . "`";
     }
 
@@ -2203,7 +2231,7 @@ function gen_show_status($show_alias) {
     }
 
     // Vacation CHECK
-    if ( $CONF['show_vacation'] == 'YES' ) {
+    if ( array_key_exists('show_vacation', $CONF) && $CONF['show_vacation'] == 'YES' ) {
         $stat_result = db_query_one("SELECT * FROM ". table_by_key('vacation') ." WHERE email = ? AND active = ? ", array($show_alias, db_get_boolean(true) )) ;
         if (!empty($stat_result)) {
             $stat_string .= "<span style='background-color:" . $CONF['show_vacation_color'] . "'>" . $CONF['show_status_text'] . "</span>&nbsp;";
@@ -2213,7 +2241,7 @@ function gen_show_status($show_alias) {
     }
 
     // Disabled CHECK
-    if ( $CONF['show_disabled'] == 'YES' ) {
+    if ( array_key_exists('show_disabled', $CONF) &&  $CONF['show_disabled'] == 'YES' ) {
         $stat_result = db_query_one(
             "SELECT * FROM ". table_by_key('mailbox') ." WHERE username = ? AND active = ?",
             array($show_alias, db_get_boolean(false))
@@ -2226,13 +2254,13 @@ function gen_show_status($show_alias) {
     }
 
     // Expired CHECK
-    if ( Config::bool('password_expiration') && Config::bool('show_expired') ) {
+    if (Config::has('password_expiration') && Config::bool('password_expiration') && Config::bool('show_expired')) {
         $now = 'now()';
         if (db_sqlite()) {
             $now = "datetime('now')";
         }
 
-        $stat_result = db_query_one("SELECT * FROM ". table_by_key('mailbox') ." WHERE username = ? AND password_expiry <= $now AND active = ?", array( $show_alias , db_get_boolean(true) ));
+        $stat_result = db_query_one("SELECT * FROM " . table_by_key('mailbox') . " WHERE username = ? AND password_expiry <= $now AND active = ?", array($show_alias, db_get_boolean(true)));
 
         if (!empty($stat_result)) {
             $stat_string .= "<span style='background-color:" . $CONF['show_expired_color'] . "'>" . $CONF['show_status_text'] . "</span>&nbsp;";
